"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageclient_1 = require("vscode-languageclient");
const vscode_1 = require("vscode");
const helpers_1 = require("./helpers");
class CommandRegister {
    constructor(client, win = vscode_1.window, cmds = vscode_1.commands) {
        this.client = client;
        this.win = win;
        this.cmds = cmds;
        this.disposables = [];
        this.isReady = false;
    }
    register() {
        return helpers_1.tap(this, () => {
            this.registerCommand('phpunit.client.test', (command, uri, editor) => {
                this.execute({ command: `${command}.nearest`, arguments: [uri, uri, [editor.selection.active.line]] });
            });
            this.registerCommand('phpunit.client.test.file', (command, uri) => {
                this.execute({ command: command, arguments: [uri, uri, []] });
            });
            this.registerCommand('phpunit.client.test.suite', (command, uri) => {
                this.execute({ command: command, arguments: [uri, '', []] });
            });
            this.registerCommand('phpunit.client.test.nearest', (command, uri, editor) => {
                this.execute({ command: command, arguments: [uri, uri, [editor.selection.active.line]] });
            });
            this.registerCommand('phpunit.client.test.last', (command, uri) => {
                this.execute({ command: command, arguments: [uri, uri, []] });
            });
        });
    }
    ready() {
        this.isReady = true;
        return this;
    }
    dispose() {
        return this.disposables;
    }
    registerCommand(command, cb = () => { }) {
        this.disposables.push(this.cmds.registerTextEditorCommand(command, () => {
            helpers_1.when(this.win.activeTextEditor, (editor) => {
                cb(command.replace(/^phpunit\.client/, 'phpunit'), editor.document.uri.toString(), editor);
            });
        }));
    }
    execute(command) {
        if (this.isReady === false) {
            return;
        }
        this.client.sendRequest(vscode_languageclient_1.ExecuteCommandRequest.type, Object.assign({
            title: '',
            command: '',
            arguments: ['', '', []],
        }, command));
    }
}
exports.CommandRegister = CommandRegister;
//# sourceMappingURL=command-register.js.map