/* --------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const command_register_1 = require("./command-register");
const decorate_manage_1 = require("./decorate-manage");
const vscode_1 = require("vscode");
const statusbar_manager_1 = require("./statusbar-manager");
const vscode_languageclient_1 = require("vscode-languageclient");
function activate(context) {
    // The server is implemented in node
    const serverModule = context.asAbsolutePath(path.join('server', 'server.js'));
    // The debug options for the server
    const debugOptions = { execArgv: ['--nolazy', '--inspect=6009'] };
    // If the extension is launched in debug mode then the debug server options are used
    // Otherwise the run options are used
    const serverOptions = {
        run: { module: serverModule, transport: vscode_languageclient_1.TransportKind.ipc },
        debug: { module: serverModule, transport: vscode_languageclient_1.TransportKind.ipc, options: debugOptions },
    };
    // Options to control the language client
    const clientOptions = {
        // Register the server for plain text documents
        documentSelector: [{ scheme: 'file', language: 'php' }],
        synchronize: {
            // Synchronize the setting section 'languageServerExample' to the server
            configurationSection: 'phpunit',
            // Notify the server about file changes to '.clientrc files contain in the workspace
            fileEvents: vscode_1.workspace.createFileSystemWatcher('**/*.php'),
        },
        outputChannel: vscode_1.window.createOutputChannel('PHPUnit'),
        revealOutputChannelOn: vscode_languageclient_1.RevealOutputChannelOn.Error,
        diagnosticCollectionName: 'PHPUnit',
    };
    const client = new vscode_languageclient_1.LanguageClient('phpunit', 'PHPUnit Language Server', serverOptions, clientOptions);
    const decorateManager = new decorate_manage_1.DecorateManager(client, context, vscode_1.window);
    const commandRegister = new command_register_1.CommandRegister(client, vscode_1.window).register();
    const statusBarManager = new statusbar_manager_1.StatusBarManager(client);
    client.onReady().then(() => {
        commandRegister.ready();
        decorateManager.listen();
        statusBarManager.listen();
    });
    // Create the language client and start the client.
    const disposable = client.start();
    // Push the disposable to the context's subscriptions so that the
    // client can be deactivated on extension deactivation
    context.subscriptions.push(disposable);
    context.subscriptions.push(commandRegister.dispose());
}
exports.activate = activate;
//# sourceMappingURL=extension.js.map