"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_1 = require("vscode");
class Spinner {
    constructor(i = 0) {
        this.i = i;
        this.frames = ['⠋', '⠙', '⠹', '⠸', '⠼', '⠴', '⠦', '⠧', '⠇', '⠏'];
    }
    frame() {
        return this.frames[(this.i = ++this.i % this.frames.length)];
    }
}
class StatusBarManager {
    constructor(client, win = vscode_1.window, spinner = new Spinner()) {
        this.client = client;
        this.win = win;
        this.spinner = spinner;
        this.statusKey = 'PHPUnit';
        this.statusBarItem = this.win.createStatusBarItem(vscode_1.StatusBarAlignment.Left);
        this.initial();
    }
    listen() {
        this.stopped();
        this.statusBarItem.show();
        this.client.onNotification('running', this.onRunning.bind(this));
        this.client.onNotification('done', this.onDone.bind(this));
    }
    onRunning() {
        this.client.outputChannel.clear();
        this.running();
    }
    onDone(state) {
        if (state.failed > 0) {
            this.failed();
        }
        else if (state.warning > 0) {
            this.failed();
        }
        else if (state.passed > 0) {
            this.success();
        }
        else {
            this.stopped();
        }
    }
    initial() {
        this.updateStatus('...');
    }
    running(details) {
        clearInterval(this.statusBarSpinner);
        this.statusBarSpinner = setInterval(() => {
            this.statusBarItem.text = `${this.statusKey} ${this.spinner.frame()} ${details || ''}`;
        }, 100);
    }
    success(details) {
        this.updateStatus('$(check)', details);
    }
    failed(details) {
        this.updateStatus('$(alert)', details);
    }
    stopped(details) {
        this.updateStatus('$(stopped)', details);
    }
    updateStatus(message, details) {
        clearInterval(this.statusBarSpinner);
        this.statusBarItem.text = `${this.statusKey} ${message} ${details || ''}`;
    }
}
exports.StatusBarManager = StatusBarManager;
//# sourceMappingURL=statusbar-manager.js.map