"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path_1 = require("path");
const os_1 = require("os");
const vscode_uri_1 = require("vscode-uri");
class Common {
    constructor() {
        this.systemPaths = [];
        this.delimiter = '/';
        this.extensions = [];
    }
    exists(path) {
        return new Promise(resolve => {
            fs_1.stat(this.normalizePath(path), err => {
                resolve(err && err.code === 'ENOENT' ? false : true);
            });
        });
    }
    get(path) {
        return new Promise((resolve, reject) => {
            return fs_1.readFile(this.normalizePath(path), (err, buffer) => {
                err ? reject(err) : resolve(buffer.toString('utf8'));
            });
        });
    }
    getSystemPaths() {
        return this.systemPaths;
    }
    where(search, cwd = process.cwd()) {
        return __awaiter(this, void 0, void 0, function* () {
            const paths = [cwd].concat(this.getSystemPaths());
            const extensions = this.extensions;
            for (const path of paths) {
                for (const ext of extensions) {
                    const file = path_1.resolve(path, `${search}${ext}`);
                    if ((yield this.exists(file)) === true) {
                        return file;
                    }
                }
            }
            return '';
        });
    }
    which(search, cwd = process.cwd()) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.where(search, cwd);
        });
    }
    findUp(search, cwd = process.cwd(), root) {
        return __awaiter(this, void 0, void 0, function* () {
            let file;
            root = path_1.resolve(!root ? path_1.parse(cwd).root : root);
            cwd = path_1.resolve(cwd);
            do {
                file = path_1.resolve(cwd, search);
                if ((yield this.exists(file)) === true) {
                    return file;
                }
                if (cwd === root) {
                    break;
                }
                cwd = path_1.resolve(cwd, '..');
            } while (cwd !== root);
            file = path_1.resolve(cwd, search);
            return (yield this.exists(file)) === true ? file : '';
        });
    }
    setSystemPaths(systemPaths) {
        this.systemPaths = systemPaths
            .split(new RegExp(this.delimiter, 'g'))
            .map((path) => path.replace(new RegExp(`${this.delimiter}$`, 'g'), '').trim());
        return this;
    }
    dirname(path) {
        return path_1.dirname(path);
    }
    tmpfile(extension = 'tmp', prefix) {
        prefix = prefix ? `${prefix}-` : '';
        return path_1.resolve(os_1.tmpdir(), `${prefix}${new Date().getTime()}.${extension}`);
    }
    unlink(path) {
        return new Promise(resolve => fs_1.unlink(path, (error) => resolve(error ? false : true)));
    }
    uri(path) {
        return this.isUri(path) === true ? vscode_uri_1.default.parse(path).toString() : vscode_uri_1.default.file(path).toString();
    }
    isUri(uri) {
        return /^file:\/\//.test(uri);
    }
}
exports.Common = Common;
//# sourceMappingURL=common.js.map