"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const php_parser_1 = require("php-parser");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
class Ast {
    parse(code, uri) {
        return this.getTestNodes(php_parser_1.default.parseCode(code, {
            ast: {
                withPositions: true,
                withSource: true,
            },
            parser: {
                debug: false,
                extractDoc: true,
                suppressErrors: true,
            },
            lexer: {
                all_tokens: true,
                comment_tokens: true,
                mode_eval: true,
                asp_tags: true,
                short_tags: true,
            },
        }), uri);
    }
    getTestNodes(node, uri) {
        return node.children.reduce((classes, nsOrClass) => {
            const namespace = nsOrClass.kind === 'namespace' ? nsOrClass.name : '';
            return (nsOrClass.kind === 'namespace' ? classes.concat(nsOrClass.children) : classes.concat(nsOrClass))
                .filter((o) => this.isClass(o))
                .reduce((c, o) => c.concat(this.convertToTestNodes(o, uri, namespace)), []);
        }, []);
    }
    convertToTestNodes(node, uri, namespace) {
        const oClass = namespace ? `${namespace}\\${node.name}` : node.name;
        const classname = oClass.replace(/\\/g, '.');
        const methods = node.body
            .filter((method) => this.isTestMethod(method))
            .map((method) => this.convertToTestNode(method, uri, oClass, classname));
        return methods.length === 0 ? [] : [this.convertToTestNode(node, uri, oClass, classname)].concat(methods);
    }
    convertToTestNode(node, uri, oClass, classname) {
        const { start } = node.loc;
        return {
            class: oClass,
            classname: classname,
            name: node.kind === 'method' ? node.name : oClass,
            uri: uri,
            range: vscode_languageserver_types_1.Range.create(start.line - 1, start.column, start.line - 1, start.column + node.name.length),
        };
    }
    isClass(node) {
        return node.kind === 'class' && node.isAbstract === false;
    }
    isTestMethod(node) {
        return (node.isAbstract === false &&
            node.kind === 'method' &&
            // /markTest(Skipped|Incomplete)/.test(node.body.loc.source) === false &&
            (/^test/.test(node.name) === true ||
                (node.leadingComments &&
                    node.leadingComments.some((comment) => /@test/.test(comment.value)) === true)));
    }
}
exports.Ast = Ast;
//# sourceMappingURL=ast.js.map