"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const filesystem_1 = require("../filesystem");
const junit_1 = require("./junit");
const helpers_1 = require("../helpers");
const parameters_1 = require("./parameters");
const process_1 = require("./process");
const events_1 = require("events");
class Cli {
    constructor(files = new filesystem_1.Filesystem(), process = new process_1.Process(), parameters = new parameters_1.Parameters(), jUnit = new junit_1.JUnit(), dispatcher = new events_1.EventEmitter()) {
        this.files = files;
        this.process = process;
        this.parameters = parameters;
        this.jUnit = jUnit;
        this.dispatcher = dispatcher;
        this.binary = '';
        this.defaults = [];
        this.output = '';
        this.tests = [];
    }
    setBinary(binary) {
        return helpers_1.tap(this, () => {
            this.binary = binary;
        });
    }
    setDefault(args) {
        return helpers_1.tap(this, () => {
            this.defaults = args;
        });
    }
    run(path, params = [], cwd = process.cwd()) {
        return __awaiter(this, void 0, void 0, function* () {
            if (path !== '') {
                path = this.files.normalizePath(path);
                cwd = this.files.dirname(path);
            }
            const root = yield this.getRoot(cwd);
            const command = {
                title: '',
                command: yield this.getBinary(cwd, root),
                arguments: yield this.getArguments(params, path, cwd, root),
            };
            this.dispatcher.emit('running', path, params, command);
            this.output = yield this.process.spawn(command);
            this.tests = yield this.jUnit.parseFile(this.parameters.get('--log-junit'));
            this.dispatcher.emit('done', this.output, this.tests, path, params, command);
            return 0;
        });
    }
    getOutput() {
        return this.output;
    }
    getTests() {
        return this.tests;
    }
    on(eventName, cb) {
        return helpers_1.tap(this, () => this.dispatcher.on(eventName, cb));
    }
    once(eventName, cb) {
        return helpers_1.tap(this, () => this.dispatcher.once(eventName, cb));
    }
    getRoot(cwd) {
        return __awaiter(this, void 0, void 0, function* () {
            return helpers_1.value(yield this.files.findUp('composer.json', cwd), (composerPath) => {
                return composerPath ? this.files.dirname(composerPath) : cwd;
            });
        });
    }
    getBinary(cwd, root) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.binary) {
                return this.binary;
            }
            return ((yield this.files.findUp(`vendor/bin/phpunit${helpers_1.os() === helpers_1.OS.WIN ? '.bat' : ''}`, cwd, root)) ||
                (yield this.files.which('phpunit')));
        });
    }
    getArguments(params, path, cwd, root) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.parameters
                .setCwd(cwd)
                .setRoot(root)
                .set(this.defaults.concat(params.concat([path]).filter((item) => !!item)))
                .all();
        });
    }
}
exports.Cli = Cli;
//# sourceMappingURL=cli.js.map