"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const filesystem_1 = require("../filesystem");
const helpers_1 = require("../helpers");
class Parameters {
    constructor(files = new filesystem_1.Filesystem()) {
        this.files = files;
        this.arguments = [];
        this.cwd = '';
        this.root = '';
        this.jUnitDotXml = '';
    }
    set(args) {
        return helpers_1.tap(this, (self) => {
            self.arguments = args;
        });
    }
    get(property) {
        const index = this.arguments.indexOf(property);
        if (index !== -1) {
            if (index === this.arguments.length - 1) {
                return true;
            }
            const value = this.arguments[index + 1];
            return value.indexOf('-') === 0 ? true : value;
        }
        return false;
    }
    exists(property) {
        return this.arguments.some((arg) => property === arg);
    }
    setCwd(cwd) {
        return helpers_1.tap(this, (self) => {
            self.cwd = cwd;
        });
    }
    setRoot(root) {
        return helpers_1.tap(this, (self) => {
            self.root = root;
        });
    }
    all() {
        return __awaiter(this, void 0, void 0, function* () {
            const phpUnitDotXml = yield this.findPhpUnitDotXml();
            if (phpUnitDotXml) {
                this.arguments = this.arguments.concat(['-c', phpUnitDotXml]);
            }
            if (this.exists('--log-junit') === false) {
                this.jUnitDotXml = this.files.tmpfile('xml', 'phpunit-lsp');
                this.arguments = this.arguments.concat(['--log-junit', this.jUnitDotXml]);
            }
            return this.arguments;
        });
    }
    findPhpUnitDotXml() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.exists('-c') === true || this.exists('--configuration') === true) {
                return '';
            }
            return ((yield this.files.findUp('phpunit.xml', this.cwd, this.root)) ||
                (yield this.files.findUp('phpunit.xml.dist', this.cwd, this.root)));
        });
    }
}
exports.Parameters = Parameters;
//# sourceMappingURL=parameters.js.map