"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const filesystem_1 = require("../filesystem");
const vscode_languageserver_types_1 = require("vscode-languageserver-types");
class TextlineRange {
    constructor(files = new filesystem_1.Filesystem()) {
        this.files = files;
        this.items = new Map();
    }
    create(uri, lineAt) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.createRange(yield this.getLines(uri), lineAt);
        });
    }
    findMethod(uri, lineAt) {
        return __awaiter(this, void 0, void 0, function* () {
            const lines = yield this.getLines(uri);
            while (lineAt > 0) {
                const line = lines[lineAt];
                const match = line.match(/^\s*(?:public|private|protected)?\s*function\s*(\w+)\s*\(.*$/);
                if (match) {
                    return match[1];
                }
                lineAt = lineAt - 1;
            }
            return '';
        });
    }
    clear() {
        this.items.clear();
        return this;
    }
    createRange(lines, lineAt) {
        const line = lines[lineAt];
        const firstNonWhitespaceCharacterIndex = line.search(/\S|$/);
        return vscode_languageserver_types_1.Range.create({
            line: lineAt,
            character: firstNonWhitespaceCharacterIndex,
        }, {
            line: lineAt,
            character: firstNonWhitespaceCharacterIndex + line.trim().length,
        });
    }
    getLines(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.items.has(uri) === false) {
                const content = yield this.files.get(uri);
                this.items.set(uri, content.split(/\r?\n/g));
            }
            return this.items.get(uri) || [];
        });
    }
}
exports.TextlineRange = TextlineRange;
//# sourceMappingURL=textline-range.js.map